@@ -162,10 +162,11 @@ class NeutronNetworkPlugin(network.NetworkBaseAPI):
         device_owner = kwargs.get('device_owner', 'share')
 
         ports = []
-        for __ in range(0, allocation_count):
+        for current_count in range(0, allocation_count):
             ports.append(self._create_port(
                          context, share_server, share_network,
-                         share_network_subnet, device_owner))
+                         share_network_subnet, device_owner,
+                         current_count))
 
         return ports
 
@@ -295,7 +296,8 @@ class NeutronNetworkPlugin(network.NetworkBaseAPI):
                                                             ip_version}
         raise exception.NetworkBadConfigurationException(reason=msg)
 
-    def deallocate_network(self, context, share_server_id):
+    def deallocate_network(self, context, share_server_id,
+                           share_network=None, share_network_subnet=None):
         """Deallocate neutron network resources for the given share server.
 
         Delete previously allocated neutron ports, delete manila db
@@ -303,6 +305,8 @@ class NeutronNetworkPlugin(network.NetworkBaseAPI):
 
         :param context: RequestContext object
         :param share_server_id: id of share server
+        :param share_network: share network data
+        :param share_network_subnet: share network subnet data
         :rtype: None
         """
         ports = self.db.network_allocations_get_for_share_server(
@@ -311,19 +315,38 @@ class NeutronNetworkPlugin(network.NetworkBaseAPI):
         for port in ports:
             self._delete_port(context, port)
 
+        # It may be possible that there are ports existing without a
+        # corresponding manila network allocation entry in the manila db,
+        # because port create request may have been successfully sent to
+        # neutron, but the response, the created port could not be stored
+        # in manila due to unreachable db
+        if share_network_subnet:
+            ports = []
+            try:
+                ports = self.neutron_api.list_ports(
+                    network_id=share_network_subnet['neutron_net_id'],
+                    device_owner='manila:share',
+                    device_id=share_server_id)
+            except exception.NetworkException:
+                LOG.warning("Failed to list ports using neutron API during "
+                            "deallocate_network.")
+            for port in ports:
+                self._delete_port(context, port, ignore_db=True)
+
     def _get_port_create_args(self, share_server, share_network_subnet,
-                              device_owner):
+                              device_owner, count=0):
         return {
             "network_id": share_network_subnet['neutron_net_id'],
             "subnet_id": share_network_subnet['neutron_subnet_id'],
             "device_owner": 'manila:' + device_owner,
             "device_id": share_server.get('id'),
+            "name": share_server.get('id') + '_' + str(count)
         }
 
     def _create_port(self, context, share_server, share_network,
-                     share_network_subnet, device_owner):
+                     share_network_subnet, device_owner, count=0):
         create_args = self._get_port_create_args(
-            share_server, share_network_subnet, device_owner)
+            share_server, share_network_subnet, device_owner, count)
 
         port = self.neutron_api.create_port(
             share_network['project_id'], **create_args)
@@ -352,7 +375,7 @@ class NeutronNetworkPlugin(network.NetworkBaseAPI):
 
         return self.db.network_allocation_create(context, port_dict)
 
-    def _delete_port(self, context, port):
+    def _delete_port(self, context, port, ignore_db=False):
         try:
             self.neutron_api.delete_port(port['id'])
         except exception.NetworkException:
@@ -360,7 +383,8 @@ class NeutronNetworkPlugin(network.NetworkBaseAPI):
                 context, port['id'], {'status': constants.STATUS_ERROR})
             raise
         else:
-            self.db.network_allocation_delete(context, port['id'])
+            if not ignore_db:
+                self.db.network_allocation_delete(context, port['id'])
 
     def _has_provider_network_extension(self):
         extensions = self.neutron_api.list_extensions()
@@ -398,13 +422,13 @@ class NeutronNetworkPlugin(network.NetworkBaseAPI):
                        "(found: %s)." % (phy, phy_nets))
                 raise exception.NetworkBadConfigurationException(reason=msg)
         else:
-            network_type = net_info['provider:network_type']
-            segmentation_id = net_info['provider:segmentation_id']
+            network_type = net_info.get('provider:network_type')
+            segmentation_id = net_info.get('provider:segmentation_id')
 
         provider_nw_dict = {
             'network_type': network_type,
             'segmentation_id': segmentation_id,
-            'mtu': net_info['mtu'],
+            'mtu': net_info.get('mtu'),
         }
         share_network_subnet.update(provider_nw_dict)
 
@@ -577,10 +601,10 @@ class NeutronBindNetworkPlugin(NeutronNetworkPlugin):
         raise exception.NetworkBindException(msg)
 
     def _get_port_create_args(self, share_server, share_network_subnet,
-                              device_owner):
+                              device_owner, count=0):
         arguments = super(
             NeutronBindNetworkPlugin, self)._get_port_create_args(
-            share_server, share_network_subnet, device_owner)
+            share_server, share_network_subnet, device_owner, count)
         arguments['host_id'] = self.config.neutron_host_id
         arguments['binding:vnic_type'] = self.config.neutron_vnic_type
         if self.binding_profiles:
